var MSG_TYPE_AUDIO = 10;
var MSG_TYPE_IMAGE = 20;
var MSG_TYPE_STATE = 25;
var MSG_TYPE_TRG = 30;
var MSG_TYPE_TERM = 40;
var MSG_CMD_PLAY = 50;
var MSG_CMD_STOP = 60;
var MSG_CMD_NOT_SUPPORTED = 70;
var MSG_CMD_DATA = 80;
var MSG_CMD_CODEC_DATA = 81;
var MSG_CMD_DATA_NOT_READY = 85;

var gSocket = null;
var gInitialMessage = null;
var gHasAudioAPI = false;
var gHasAudioMoz = false;
var gAudioNode = null;
var gAudioCtx = null;
var gAudioMoz = null;
var gWSConnectionCallback = null;
var gWSImageCallback = null;
var gWSStateCallback = null;
var gWSConnectionLossCallback = null;
var gkAudioRate = 22050;
var gAudioSampleBufferLen = 22050;
var gAudioSamples = null;
var gAudioNumSamples = 0;
var gAudioSupportEnabled = true;
var gSocBytes = 0;

function checkAudioAPI() {
	if (!window.AudioContext) {
		if (!window.webkitAudioContext) {
			return false;
		}
		window.AudioContext = window.webkitAudioContext;
	}
	return true;
}

function checkAudioMoz() {
	if ('Audio' in window) {
		var a = new Audio();
		if (a.mozSetup)
			return true;
		else
			return false;
	}
	return false;
}

function checkWS() {
	if ('WebSocket' in window)
		return true;
	return false;
}

function sendMessage(msg) {
	if (gSocket)
		gSocket.send(msg);
}

function createSocket(initialWSCommand) {
	if (!checkWS())
		return false;

	if (gSocket)
		return true;

	gSocket = null;
	var prefix = window.location.pathname ? window.location.pathname : "/";
	prefix = prefix.replace("//", "/");
	var host = window.location.host;
	var wsurl = ((window.location.protocol === "https:") ? "wss://" : "ws://") + host + prefix + "ws";
	var conn = new WebSocket(wsurl);
	conn.binaryType = 'arraybuffer';
	gInitialMessage = initialWSCommand;
	conn.onopen = function() {
		gSocBytes = 0;
		conn.send( "user-agent:" + navigator.userAgent );
		if (gInitialMessage)
			conn.send(gInitialMessage);
		gInitialMessage = null;
		gSocket = conn;
		if (gWSConnectionCallback)
			gWSConnectionCallback();
	}
	conn.onclose = function(code, reason, remote) {
		gSocket = null;
		if (gWSConnectionLossCallback)
			gWSConnectionLossCallback();
	}
	conn.onerror = function(error) {
		gSocket = null;
		if (gWSConnectionLossCallback)
			gWSConnectionLossCallback();
	}
	conn.onmessage = function(msg) {
		gSocBytes += msg.data.byteLength;
		var header = new DataView(msg.data, 0, 6);
		if (header.getUint8(0) == MSG_TYPE_IMAGE) {
			if (header.getUint8(1) == MSG_CMD_DATA_NOT_READY)
				OnRecvImage(null);
			else
				OnRecvImage(msg.data);
		}
		else if (header.getUint8(0) == MSG_TYPE_AUDIO) {
			var datatype = header.getUint8(1);
			if (datatype == MSG_CMD_NOT_SUPPORTED) {
			   gAudioSupportEnabled = false;
			}
			else if (datatype == MSG_CMD_DATA){
			   gAudioSupportEnabled = true;
			   OnRecvAudio(msg.data);
			}
			else if (datatype == MSG_CMD_CODEC_DATA){
				gAudioSupportEnabled = true;
				OnRecvAudioALaw(msg.data);
			}
		}
		else if (header.getUint8(0) == MSG_TYPE_STATE) {
			if (header.getUint8(1) == MSG_CMD_DATA) {
			   OnRecvState(msg.data);
			}
		}
	}
	return true;
}

function getCountSocketBytes() {
	return gSocBytes;
}

function resetCountSocketBytes() {
	gSocBytes = 0;
}

function OnRecvImage(data) {
	if (!data) {
		if (gWSImageCallback)
			gWSImageCallback(null);
		return;
	}
	var chunk = new Int8Array(data, 6);
	var blob = new Blob([chunk], {type: "image/jpeg"});
	var reader = new FileReader();
	reader.onload = function(event){
		if (gWSImageCallback)
			gWSImageCallback(event.target.result);
		else
			$('#webpix').attr( 'src', event.target.result );
	};
	reader.readAsDataURL(blob);
}

function OnRecvAudio(data) {
	var samplebuffer = new Int16Array(data, 6);
	var len = samplebuffer.length;

	if (gHasAudioMoz) {
		if (!gAudioMoz) {
			gAudioMoz = Audio();
			gAudioMoz.mozSetup(1, gkAudioRate);
		}
		var audio = gAudioMoz;
		var samples = new Float32Array(len);
		for (i = 0; i < len; ++i) {
			samples[i] = samplebuffer[i] / 32768.0;
		}
		audio.mozWriteAudio(samples);
	}
	else if (gHasAudioAPI) {
		if (len == 0) {
			return;
		}
		if (!gAudioSamples) {
			gAudioSampleBufferLen = len;
			gAudioSamples = gAudioCtx.createBuffer(1, gAudioSampleBufferLen, gkAudioRate);
			if (!gAudioSamples) {
				return;
			}
		}
		var samples = gAudioSamples.getChannelData(0);
		for (j = 0, i = gAudioNumSamples; j < len; ++i, ++j) {
			samples[i] = samplebuffer[j] / 32768.0;
		}
		gAudioNumSamples += len;
		if (gAudioNumSamples >= gAudioSampleBufferLen) {
			gAudioNumSamples = 0;
			gAudioNode = gAudioCtx.createBufferSource();
			gAudioNode.buffer = gAudioSamples;
			gAudioNode.connect(gAudioCtx.destination);
			gAudioNode.start(0);
			gAudioSamples = null;
		}
	}
}

function decodeALaw(sample) {
	var out = 0;
	var ix = sample & 0x7f;
	var exp = ix >> 4;
	var mant = ix & 0xf;
	if (exp > 0)
		mant += 16;
	mant <<= 4;
	if (exp > 1)
		mant <<= (exp-1);
	out = (sample >= 0) ? mant : -mant;
	return out;
}

function OnRecvAudioALaw(data) {

	var samplebuffer = new Int8Array(data, 6);
	var len = samplebuffer.length;

	if (gHasAudioMoz) {
		if (!gAudioMoz) {
			gAudioMoz = Audio();
			gAudioMoz.mozSetup(1, gkAudioRate);
		}
		var audio = gAudioMoz;
		var samples = new Float32Array(len);
		for (i = 0; i < len; ++i) {
			samples[i] = decodeALaw(samplebuffer[i]) / 32768.0;
		}
		audio.mozWriteAudio(samples);
	}
	else if (gHasAudioAPI) {
		if (len == 0) {
			return;
		}
		if (!gAudioSamples) {
			gAudioSampleBufferLen = len;
			gAudioSamples = gAudioCtx.createBuffer(1, gAudioSampleBufferLen, gkAudioRate);
			if (!gAudioSamples) {
				return;
			}
		}
		var samples = gAudioSamples.getChannelData(0);
		for (j = 0, i = gAudioNumSamples; j < len; ++i, ++j) {
			samples[i] = decodeALaw(samplebuffer[j]) / 32768.0;
		}
		gAudioNumSamples += len;
		if (gAudioNumSamples >= gAudioSampleBufferLen) {
			gAudioNumSamples = 0;
			gAudioNode = gAudioCtx.createBufferSource();
			gAudioNode.buffer = gAudioSamples;
			gAudioNode.connect(gAudioCtx.destination);
			gAudioNode.start(0);
			gAudioSamples = null;
		}
	}
}

function OnRecvState(data) {
  if (gWSStateCallback) {
	 var strdata = String.fromCharCode.apply(null, new Uint8Array(data, 6));
	 gWSStateCallback(strdata);
  }
}

function setupAudioAPI() {
	if ( !gAudioCtx )
		gAudioCtx = new AudioContext();
}

function setupAudioMoz() {
}

function setupAudio(initialWSCommand) {
   if (checkAudioAPI()) {
	   gHasAudioAPI = true;
	   setupAudioAPI();
   }
   else if (checkAudioMoz()) {
	   gHasAudioMoz = true;
	   setupAudioMoz();
   }
   else {
	   return false;
   }
   return true;
}

function setWSConnectionCallback(cb) {
	gWSConnectionCallback = cb;
}

function setWSConnectionLossCallback(cb) {
	gWSConnectionLossCallback = cb;
}

function setWSImageCallback(cb) {
	gWSImageCallback = cb;
}

function setWSStateCallback(cb) {
	gWSStateCallback = cb;
}

function isWebSocketSupported() {
	return checkWS();
}

function isAudioTransportReady() {
	return (gSocket == null) ? false : true;
}

function isAudioSupportEnabled() {
	return gAudioSupportEnabled;
}

function isAudioSupportReady() {
	return checkWS() && (checkAudioAPI() || checkAudioMoz());
}

function requestImage(resultsCallback) {
	if (!gSocket)
		return false;

	if (resultsCallback)
		setWSImageCallback(resultsCallback);

	if (gSocket) {
		sendMessage('GET_IMAGE');
	}
	return true;
}

function requestState(resultsCallback) {
	if (!gSocket)
		return false;

	if (resultsCallback)
		setWSStateCallback(resultsCallback);

	if (gSocket) {
		sendMessage('GET_STATE');
	}
	return true;
}

function enableAudioTransmission(enable) {
	if (!gSocket)
		return false;
	setupAudio();
	if (enable) {
		sendMessage('START_AUDIO');
	}
	else {
		sendMessage('STOP_AUDIO');
		gAudioSamples = null;
	}
	return true;
}
